import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:getx_extended/export.dart';

abstract class RxTabPage<TAB> extends StatelessWidget {
  Widget backdrop(List<TAB> tabs) => Container();

  RxModel<List<TAB>> get tabs;

  Widget innerPage(TAB tab);

  Widget appBar(PreferredSizeWidget tabBar);

  Widget tabBuilder(TAB tab, bool isSelected) => Tab(text: "$tab");

  void onTapTabBar(int page) => {};

  Rx<int> get page => 0.obs;

  PreferredSizeWidget tabBar(
    Rx<int> page,
    List<TAB> tabs,
  ) =>
      RxTabBar<TAB>(
        page: page,
        source: tabs,
        builder: tabBuilder,
        onTap: onTapTabBar,
      );

  @override
  Widget build(BuildContext context) => Scaffold(
        body: tabs.toWidget(
          (tabs) => DefaultTabController(
            length: tabs.value.length,
            child: RxPageDetector(
              page: page,
              child: (page) => NestedScrollView(
                headerSliverBuilder: (_, __) => [
                  appBar(tabBar(page, tabs.value)),
                ],
                body: Stack(
                  children: [
                    Positioned.fill(child: backdrop(tabs.value)),
                    if (tabs.value.isNotEmpty)
                      TabBarView(
                        children: [
                          ...tabs.value.map(innerPage),
                        ],
                      ),
                  ],
                ),
              ),
            ),
          ),
        ),
      );
}
